package org.leo.traceroute.util;

import javax.swing.*;
import java.awt.*;

public class CheckThreadViolationRepaintManager extends RepaintManager {

    public CheckThreadViolationRepaintManager() {
        super();
        setDoubleBufferingEnabled(true);
    }

    public synchronized void addInvalidComponent(JComponent component) {
        checkThreadViolations(component);
        super.addInvalidComponent(component);
    }

    public void addDirtyRegion(JComponent component, int x, int y, int w, int h) {
        checkThreadViolations(component);
        super.addDirtyRegion(component, x, y, w, h);
    }

    @Override
    public void addDirtyRegion(Window window, int x, int y, int w, int h) {
        checkThreadViolations(window);
        super.addDirtyRegion(window, x, y, w, h);
    }

    private void checkThreadViolations(Component c) {
        if (!SwingUtilities.isEventDispatchThread() && (c.isShowing())) {
            Exception exception = new Exception();
            boolean repaint = false;
            boolean fromSwing = false;
            StackTraceElement[] stackTrace = exception.getStackTrace();
            for (StackTraceElement st : stackTrace) {
                if (repaint && st.getClassName().startsWith("javax.swing.")) {
                    fromSwing = true;
                }
                if ("repaint".equals(st.getMethodName())) {
                    repaint = true;
                }
            }
            if (repaint && !fromSwing) {
                // no problems here, since repaint() is thread safe
                return;
            }
            exception.printStackTrace();
        }
    }
}
    